open Printf

type program = Program of string * vardec list * istmt list 
and vardec = info * string * ttype
and istmt = info * stmt
and stmt =  
    Assign of string * iexpr
  | Read of string
  | Write of iexpr
  | Sequence of istmt list
  | IfThenElse of iexpr * istmt * istmt option
  | While of iexpr * istmt
  | For of string * iexpr * iexpr * int * istmt
and ttype = 
    TInteger
  | TReal
  | TBoolean
  | TUndef
  | TError
and iexpr = info * expr
and expr = 
    CInteger of int
  | CReal of float
  | CBoolean of bool
  | Var of string 
  | Par of iexpr
  | Op of ioperator * iexpr list
and ioperator = info * operator
and operator =
  | OAdd | OSub | OMul | OFrac | ODiv | OMod  
  | OPos | ONeg 
  | ONot | OAnd | OOr  
  | OEq  | ONeq | OLess | OLeq | OGrea | OGeq
and loc = { line: int; col: int }
and info = { locL: loc; locR: loc; typ: ttype }

let cinfo (a,b,t) = { locL=a.locL; locR=b.locR; typ=t }

let setType t r = { locL=r.locL; locR=r.locR; typ=t }

let gI (inf,_) = inf
let gE (_,e) = e

let symtab = Hashtbl.create 10007

let add_vartype ((s:string),(t:ttype)) = Hashtbl.add symtab s t
  
let errMessages = ref [] 

let print_Loc loc = sprintf "(%d,%d)" loc.line loc.col

let print_Range inf = print_Loc inf.locL ^ "-" ^ print_Loc inf.locR

let addErrMsg inf msg = 
  errMessages := !errMessages @ [ print_Range inf ^ ": " ^ msg ] 
                                         
 
let lookup_type (info,name) = 
  try Hashtbl.find symtab name 
  with Not_found -> addErrMsg info ("name " ^ name ^ " not found"); TUndef 

