open Defs
open Printf

let pprint_Type t =  
  match t with
    TUndef -> "undefined"
  | TInteger -> "integer"
  | TReal -> "real"
  | TBoolean -> "boolean"
  | TError -> "erroneous" 

let rec pprint_Op = function
    OAdd -> printf "+"
  | OSub -> printf "-"
  | OMul -> printf "*"
  | OFrac -> printf "/"
  | ONeg -> printf "-"

let rec pprint_Expression = function
    CInteger i -> printf "%d" i
  | CReal f -> printf "%f" f
  | Var var -> printf "%s" var 
  | Par e -> pprint_IExpression e   
  | Op (op,[a]) -> 
      pprint_Op (gE op);
      pprint_IExpression a
  | Op (op,[a;b]) ->
      printf "(";
      pprint_IExpression a;
      pprint_Op (gE op);
      pprint_IExpression b;
      printf ")"
and pprint_IExpression (_,x) = pprint_Expression x

let rec pprint_Statement = function
    Assign (var,exp) -> 
      printf "%s := " var;
      pprint_IExpression exp
  | Read var -> 
      printf "read %s" var 
  | Write exp -> 
      printf "write ";
      pprint_IExpression exp
and pprint_InfoStatement (_,s) = pprint_Statement s

let pprint_Program = function
    Program (_,_,x::xs) -> 
      printf "begin\n";
      List.fold_left (fun _ s -> printf ";\n"; pprint_InfoStatement s)
                     (pprint_InfoStatement x)
                     xs;
      printf "\nend\n"
